@@ -7,9 +7,8 @@
 #'  generalized eta squared, and
 #'  \code{\link[bayestestR:bayesfactor_inclusion]{inclusion Bayes factor based
 #'  on matched models}} (BFs).
-#'@param data_per_subject Data frame or name of data frame as string. Should
-#'  contain all values (measurements/observations) in a single row per each
-#'  subject.
+#'@param data_per_subject Data frame. Should contain all values
+#'  (measurements/observations) in a single row per each subject.
 #'@param values Vector of strings; column name(s) in the \code{data_per_subject}
 #'  data frame. Each column should contain a single dependent variable: thus, to
 #'  test repeated (within-subject) measurements, each specified column should
@@ -38,13 +37,27 @@
 #'  variable (representing between-subject factors).
 #'@param ci Numeric; confidence level for returned CIs. (Default: \code{.9};
 #'  Lakens, 2014; Steiger, 2004.)
+#'@param norm_tests Normality tests for the pooled ANOVA residuals
+#'  (\code{"none"} by default, giving no tests). Any or all of the following
+#'  character input is accepted (as a single string or a character vector;
+#'  case-insensitive): \code{"W"} (Shapiro-Wilk), \code{"K2"}
+#'  (D'Agostino-Pearson), \code{"A2"} (Anderson-Darling), \code{"JB"}
+#'  (Jarque-Bera); see \code{\link{norm_tests}}. The option \code{"all"} selects
+#'  all four previous tests at the same time.
+#'@param norm_plots If \code{TRUE} and \code{norm_tests} is not \code{"none"},
+#'  displays density, histogram, and Q-Q plots (and scatter plots for paired
+#'  tests) for the pooled residuals.#'
+#'@param var_tests Logical, \code{FALSE} by default. If \code{TRUE} (and there
+#'  are any between-subject factors), runs variance equality tests via
+#'  \code{\link{var_tests}} for all combinations of the between-subject factors
+#'  within each level of within-subject factor combinations; see Details.
 #'@param bf_added Logical. If \code{TRUE} (default), inclusion Bayes factor is
 #'  calculated and displayed. (Note: with multiple factors and/or larger
 #'  dataset, the calculation can take considerable time.)
 #'@param bf_sample Number of samples used to estimate Bayes factor (\code{10000}
 #'  by default).
-#'@param test_title String, \code{"--- neat ANOVA ---"} by default. Simply
-#'  displayed in printing preceding the statistics.
+#'@param test_title String, \code{"--- neat ANOVA ---"} by default. Simply displayed
+#'  in printing preceding the statistics.
 #'@param welch If \code{TRUE} (default), calculates Welch's ANOVA via
 #'  \code{\link[stats:oneway.test]{stats::oneway.test}} in case of a single
 #'  factor (one-way) between-subject design. If \code{FALSE}, calculates via
@@ -60,7 +73,21 @@
 #'  larger than \code{.75}, while Huynh-Feldt correction is applied when
 #'  Mauchly's sphericity test is significant and the Greenhouse-Geisser epsilon
 #'  is larger than \code{.75} (see Girden, 1992).
+#'@param type Sum of squares type specified as a number: \code{1}, \code{2}, or
+#'  \code{3}. Set to \code{2} by default (which is generally recommended, see
+#'  e.g. Navarro, 2019, Chapter 16).
+#'@param white.adjust If not \code{FALSE} (default) uses a
+#'  heteroscedasticity-corrected coefficient covariance matrix; the various
+#'  values of the argument specify different corrections (\code{"hc0"},
+#'  \code{"hc1"}, \code{"hc2"}, \code{"hc3"}, or \code{"hc4"}). See the
+#'  documentation for \code{\link[car:hccm]{car::hccm}} for details. If set to
+#'  \code{TRUE} then the \code{"hc3"} correction is selected.
 #'@param hush Logical. If \code{TRUE}, prevents printing any details to console.
+#'@param plot_means Logical (\code{FALSE} by default). If \code{TRUE}, creates
+#'  plots of means by factor, by passing data and factor information to
+#'  \code{\link{plot_neat}}.
+#'@param ... Any additional arguments will be passed on to
+#'  \code{\link{plot_neat}}.
 #'
 #'@details
 #'
@@ -74,31 +101,47 @@
 #'original full BF number is available in the returned named vector as
 #'\code{bf}.)
 #'
-#'Levene's test is returned for between-subject design without Welch's
-#'correction, while Mauchly's sphericity test is returned for repeated measures
-#'with more than two levels. If Mauchly's test is significant, epsilon
-#'correction may be applied (see the \code{e_correction} parameter). If Levene's
-#'test is significant and the data is unbalanced (unequal group sample sizes),
-#'you should either consider the results respectively or choose a different
-#'test.
+#'Mauchly's sphericity test is returned for repeated measures with more than two
+#'levels. If Mauchly's test is significant, epsilon correction may be applied
+#'(see the \code{e_correction} parameter).
+#'
+#'Variance equality tests (if \code{var_tests} is \code{TRUE}): Brown-Forsythe
+#'and Fligner-Killeen tests are performed for each within-subject factor level
+#'(or, in case of several factors, each combination) via
+#'\code{\link{var_tests}}. Note that variance testing is generally not
+#'recommended (see e.g., Zimmerman, 2004). In case of a one-way between-subject
+#'ANOVA, Welch-corrected results are reported by default, which corrects for
+#'unequal variances. In case of multiple between-subject factors, the
+#'\code{white.adjust} parameter can be set to \code{TRUE} (or \code{"hc3"}) to
+#'apply \code{"hc3"} correction for unequal variances. In case of a mixed ANOVA
+#'(both between-subject and within-subject factors), if the tests are
+#'significant and the data is unbalanced (unequal group sample sizes), you
+#'should either consider the results respectively or choose a different test.
+#'
+#'In case of multiple values (column names) that match identical levels for all
+#'factors, the given columns will be merged into a single column taking the mean
+#'value of all these columns. (This is to simplify "dropping" a within-subject
+#'factor and retesting the remaining factors.) Explicit warning messages are
+#'shown in each such case.
 #'
 #'@return Prints ANOVA statistics (including, for each model, F-test with
 #'  partial eta squared and its CI, generalized eta squared, and BF, as
 #'  specified via the corresponding parameters) in APA style. Furthermore, when
-#'  assigned, returns a list with up to three elements. First,
+#'  assigned, returns a list with up to four elements. First,
 #'  '\code{stat_list}', a list of named vectors per each effect (main or
 #'  interaction). Each vector contains the following elements: \code{F} (F
 #'  value), \code{p} (p value), \code{petas} (partial eta squared), \code{getas}
 #'  (generalized eta squared), \code{epsilon} (epsilon used for correction), and
 #'  \code{bf} (inclusion BF; when \code{bf_added} is not \code{FALSE}). Second,
-#'  the \code{\link[ez]{ezANOVA}} object, named \code{ez_anova} (calculated even
-#'  when \code{\link[stats]{oneway.test}} is printed). Third, when
-#'  \code{bf_added} is not \code{FALSE}, the \code{\link[BayesFactor]{anovaBF}}
-#'  object, named \code{bf_models}; including all models on which the inclusion
-#'  BFs are based.
+#'  a dataframe with mean and SD per each condition (i.e., per each level per
+#'  each factor). Third, the \code{\link[ez]{ezANOVA}} object, named
+#'  \code{ez_anova} (calculated even when \code{\link[stats]{oneway.test}} is
+#'  printed). Fourth, when \code{bf_added} is not \code{FALSE}, the
+#'  \code{\link[BayesFactor]{anovaBF}} object, named \code{bf_models}; including
+#'  all models on which the inclusion BFs are based.
 #'
-#'@note All F-tests are calculated via \code{\link[ez:ezANOVA]{ez::ezANOVA}},
-#'  including Levene's test and Mauchly's sphericity test. (But Welch's ANOVA is
+#'@note The F-tests are calculated via \code{\link[ez:ezANOVA]{ez::ezANOVA}},
+#'  including Mauchly's sphericity test. (But Welch's ANOVA is
 #'  calculated in case of one-way between-subject designs via
 #'  \code{\link[stats:oneway.test]{stats::oneway.test}}, unless the \code{welch}
 #'  parameter is set to \code{FALSE}.)
@@ -138,11 +181,15 @@
 #'in JASP [Blog post]. Retrieved from
 #'\url{https://www.cogsci.nl/blog/interpreting-bayesian-repeated-measures-in-jasp}
 #'
-#'McDonald, J. H. 2015. Handbook of Biological Statistics (3rd ed.). Sparky House Publishing, Baltimore, Maryland. Retrieved from \url{http://www.biostathandbook.com}
+#'McDonald, J. H. 2015. Handbook of Biological Statistics (3rd ed.). Sparky
+#'House Publishing, Baltimore, Maryland. Retrieved from
+#'\url{http://www.biostathandbook.com}
 #'
-#'Moder, K. (2010). Alternatives to F-test in one way ANOVA in case of heterogeneity of variances (a simulation study). Psychological Test and Assessment Modeling, 52(4), 343-353.
+#'Moder, K. (2010). Alternatives to F-test in one way ANOVA in case of
+#'heterogeneity of variances (a simulation study). Psychological Test and
+#'Assessment Modeling, 52(4), 343-353.
 #'
-#'Navarro, D. (2013). Learning Statistics with R: A Tutorial for Psychology
+#'Navarro, D. (2019). Learning Statistics with R: A Tutorial for Psychology
 #'Students and Other Beginners (Version 0.6.1). Retrieved from
 #'\url{https://learningstatisticswithr.com/}
 #'
@@ -154,6 +201,11 @@
 #'Psychological Methods, 9(2), 164-182.
 #'\doi{https://doi.org/10.1037/1082-989X.9.2.164}
 #'
+#'
+#'Zimmerman, D. W. (2004). A note on preliminary tests of equality of variances.
+#'British Journal of Mathematical and Statistical Psychology, 57(1), 173–181.
+#'https://doi.org/10.1348/000711004849222
+#'
 #' @seealso \code{\link{plot_neat}}, \code{\link{t_neat}}
 #' @examples
 #' # assign random data in a data frame for illustration
@@ -250,6 +302,34 @@
 #'     bf_added = FALSE
 #' )
 #'
+#' # same as above, but now creating a plot of means
+#' # y_title passed add an example title (label) for the Y axis
+#' anova_neat(
+#'     dat_1,
+#'     values = c('value_1_a', 'value_2_a', 'value_1_b', 'value_2_b'),
+#'     within_ids = list(
+#'         letters = c('_a', '_b'),
+#'         numbers =  c('_1', '_2')
+#'     ),
+#'     between_vars = 'grouping2',
+#'     bf_added = FALSE,
+#'     plot_means = TRUE,
+#'     y_title = 'Example Y Title'
+#' )
+#'
+#' # same as above, but collapsing means over the removed "numbers" factor
+#' anova_neat(
+#'     dat_1,
+#'     values = c('value_1_a', 'value_2_a', 'value_1_b', 'value_2_b'),
+#'     within_ids = list(
+#'         letters = c('_a', '_b')
+#'     ),
+#'     between_vars = 'grouping2',
+#'     bf_added = FALSE,
+#'     plot_means = TRUE,
+#'     y_title = 'Example Y Title'
+#' )
+#'
 #' # In real datasets, these could of course be more meaningful. For example, let's
 #' # say participants rated the attractiveness of pictures with low or high levels
 #' # of frightening and low or high levels of disgusting qualities. So there are
@@ -326,18 +406,19 @@ anova_neat = function(data_per_subject,
                       within_ids = NULL,
                       between_vars = NULL,
                       ci = 0.90,
-                      bf_added = TRUE,
+                      norm_tests = 'none',
+                      norm_plots = FALSE,
+                      var_tests = FALSE,
+                      bf_added = FALSE,
                       bf_sample = 10000,
                       test_title = "--- neat ANOVA ---",
                       welch = TRUE,
                       e_correction = NULL,
-                      hush = FALSE) {
-    if (class(data_per_subject) == "character") {
-        data_wide = eval(parse(text = data_per_subject))
-        data_per_subject = data_wide
-    } else {
-        data_wide = data_per_subject
-    }
+                      type = 2,
+                      white.adjust = FALSE,
+                      hush = FALSE,
+                      plot_means  = FALSE,
+                      ...) {
     validate_args(
         match.call(),
         list(
@@ -346,15 +427,21 @@ anova_neat = function(data_per_subject,
             val_arg(within_ids, c('null', 'char', 'list'), 1),
             val_arg(between_vars, c('null', 'char')),
             val_arg(ci, c('num'), 1),
+            val_arg(norm_tests, c('char')),
+            val_arg(norm_plots, c('bool'), 1),
+            val_arg(var_tests, c('bool')),
             val_arg(bf_added, c('bool'), 1),
             val_arg(bf_sample, c('num'), 1),
             val_arg(test_title, c('char'), 1),
             val_arg(welch, c('bool'), 1),
             val_arg(e_correction, c('null', 'char'), 1, c('gg', 'hf', 'none')),
+            val_arg(type, c('num'), 1),
+            val_arg(white.adjust, c('bool', 'char'), 1),
             val_arg(hush, c('bool'), 1)
         )
     )
     cols_notfound = c()
+    bv_copy = between_vars
     if (!is.null(between_vars)) {
         for (colname in between_vars) {
             if (!colname %in% names(data_per_subject)) {
@@ -384,10 +471,53 @@ anova_neat = function(data_per_subject,
             )
         }
     }
-    val_wi_id(match.call(), within_ids, values)
+    val_levels = val_wi_id(match.call(), within_ids, values)
+    # collapsing
+    fac_dups = unique(val_levels[duplicated(val_levels)])
+    if (length(fac_dups) > 0) {
+        message('Columns with identical factors were found!',
+                ' Make sure this is how you want it:')
+        for (dup in fac_dups) {
+            to_collapse = names(val_levels)[val_levels == dup]
+            message(
+                'The columns "',
+                paste(to_collapse, collapse = '", "'),
+                '" were collapsed (averaged) into one column.'
+            )
+            data_per_subject[[dup]] = rowMeans(data_per_subject[, to_collapse], na.rm =
+                                                      TRUE)
+            values = values[!(values %in% to_collapse)]
+            values = c(values, dup)
+        }
+    }
+    # end collapsing
+    if (plot_means == TRUE) {
+        means_plot =
+            plot_neat(
+                data_per_subject,
+                values,
+                within_ids,
+                bv_copy,
+                numerics = FALSE,
+                hush = TRUE,
+                ...
+            )
+        if (!is.null(means_plot[1])) {
+            graphics::plot(means_plot)
+        }
+    }
+    dat_tot = plot_neat(
+        data_per_subject,
+        values,
+        within_ids,
+        bv_copy,
+        eb_method = stats::sd,
+        numerics = TRUE
+    )
     if (is.null(e_correction)){
         e_correction = ''
     }
+    data_wide = data_per_subject
     name_taken('within_factor', data_wide)
     name_taken('neat_unique_values', data_wide)
     name_taken('neat_unique_id', data_wide)
@@ -395,6 +525,7 @@ anova_neat = function(data_per_subject,
     data_wide[[id_col]] = as.character(seq.int(nrow(data_wide)))
     w_anova = NULL
     if (length(values) > 1) {
+        withname = "within_factor"
         data_reshaped = stats::reshape(
             data_wide,
             direction = 'long',
@@ -413,9 +544,14 @@ anova_neat = function(data_per_subject,
                 data_reshaped[[fact_name]] = as.factor(data_reshaped[[fact_name]])
             }
             within_vars = paste(names(within_ids), collapse = ', ')
-        } else if (is.character(within_ids)) {
+        } else if (is.list(within_ids)) {
+            within_vars = names(within_ids)
+            names(data_reshaped)[names(data_reshaped) == 'within_factor'] = names(within_ids)
+            withname = names(within_ids)
+        }  else if (is.character(within_ids)) {
             within_vars = within_ids
             names(data_reshaped)[names(data_reshaped) == 'within_factor'] = within_ids
+            withname = within_ids
         } else {
             within_vars = 'within_factor'
         }
@@ -439,6 +575,8 @@ anova_neat = function(data_per_subject,
     }
 
     this_data[, id_col] = to_fact(this_data[[id_col]])
+
+
     if (is.null(between_vars)) {
         between_vars_ez = 'NULL'
         between_vars_bf = ''
@@ -474,7 +612,11 @@ anova_neat = function(data_per_subject,
         between_vars_ez,
         ', within =',
         within_vars_ez,
-        ', type = 2, detailed = TRUE )'
+        ', type = ',
+        type,
+        ', white.adjust = ',
+        white.adjust,
+        ', detailed = TRUE, return_aov = TRUE)'
     )
     ez_anova_out = eval(parse(text = a_text))
     # suppressWarnings
@@ -509,6 +651,45 @@ anova_neat = function(data_per_subject,
         bf_inc = NULL
         bf_models = NULL
     }
+    resids = ez_anova_out$aov$residuals
+    if (is.null(resids)) {
+        aov_proj = stats::proj(ez_anova_out$aov)
+        resids = aov_proj[[length(aov_proj)]][, "Residuals"]
+        fitt = stats::fitted(ez_anova_out$aov[[length(ez_anova_out$aov)]])
+    } else {
+        fitt = ez_anova_out$aov$fitted
+    }
+    if (norm_tests  != 'none' &
+        hush == FALSE) {
+        prnt('--- Normality of the Residuals ---')
+        norm_tests_in(
+            var1 = resids,
+            var2 = NULL,
+            pair = FALSE,
+            norm_tests = norm_tests,
+            alpha = 0.05,
+            hush = FALSE,
+            plots = norm_plots,
+            tneet = FALSE,
+            nonparametric = FALSE,
+            aspect_ratio = 1,
+            anov = TRUE
+        )
+    }
+    if (var_tests == TRUE && !is.null(bv_copy) &
+        hush == FALSE
+    ) {
+        prnt('--- Equality of Variances ---')
+        if (is.null(within_vars)) {
+            neatStats::var_tests(value_col, bv_copy, this_data)
+        } else {
+            for (fact in unique(this_data[[withname]])) {
+                cat(fact, ': ', sep = '', fill = TRUE)
+                neatStats::var_tests(value_col, bv_copy,
+                          this_data[this_data[[withname]] == fact, ])
+            }
+        }
+    }
     to_return = anova_apa(
         ezANOVA_out = ez_anova_out,
         ci = ci,
@@ -517,7 +698,10 @@ anova_neat = function(data_per_subject,
         test_title = test_title,
         welch = w_anova,
         e_correction = e_correction,
-        hush = hush
+        hush = hush,
+        dat_tot = dat_tot,
+        resids = resids,
+        fitt = fitt
     )
     invisible(to_return)
 }
@@ -529,26 +713,10 @@ anova_apa = function(ezANOVA_out,
                      test_title = "--- neat ANOVA ---",
                      welch = NULL,
                      e_correction = '',
-                     hush = FALSE) {
-    levene = ezANOVA_out$"Levene's Test for Homogeneity of Variance"
-    ezANOVA_out$ANOVA$'p<.05' = NULL
-    if ((!is.null(levene)) && (is.null(welch))) {
-        if ('Levene' %in%  ezANOVA_out$ANOVA$Effect) {
-            stop(
-                'Sorry, the name "Levene" is reserved for this function. Remove or rename this factor.'
-            )
-        }
-        levene_post = ''
-        levene$'p<.05' = NULL
-        levene$ges = NA
-        levene = data.frame(Effect = "Levene", levene)
-        if (round(levene$p, 3) < 0.05) {
-            levene_pre = "Levene's test indicates unequal variances (p < 0.05): "
-        } else {
-            levene_pre = "Levene's test does not indicate unequal variances (p >= 0.05): "
-        }
-        ezANOVA_out$ANOVA = rbind(levene, ezANOVA_out$ANOVA)
-    }
+                     hush = FALSE,
+                     dat_tot = NULL,
+                     fitt = NULL,
+                     resids = NULL) {
     ezANOVA_out$ANOVA$pes = ezANOVA_out$ANOVA$SSn / (ezANOVA_out$ANOVA$SSn + ezANOVA_out$ANOVA$SSd)
     ezANOVA_out$ANOVA$Effect = as.character(ezANOVA_out$ANOVA$Effect)
     if (hush == FALSE) {
@@ -557,13 +725,13 @@ anova_apa = function(ezANOVA_out,
     mauchly = ezANOVA_out$"Mauchly's Test for Sphericity"
     if (!is.null(mauchly)) {
         if (hush == FALSE) {
-            prnt("- Mauchly's sphericity test:")
+            prnt("--- Mauchly's Sphericity Test ---")
         }
         eps_p_corrs = get_e_corrs(mauchly,
                                   ezANOVA_out$"Sphericity Corrections",
                                   e_correction, hush)
         if (hush == FALSE) {
-            prnt("- ANOVA:")
+            prnt("--- ANOVA ---")
         }
     } else {
         eps_p_corrs = NULL
@@ -621,55 +789,30 @@ anova_apa = function(ezANOVA_out,
         the_ci = paste0(", ", ro(ci * 100, 0), "% CI [")
         getas = ezANOVA_out$ANOVA$ges[indx]
         nG2 = sub('.', '', ro(getas, 3))
-
-        if (f_name == 'Levene') {
-            out = paste0(
-                levene_pre,
-                "F(",
-                df_n,
-                ",",
-                df_d,
-                ")",
-                " = ",
-                ro(F_val, 2),
-                ", p = ",
-                ro(pvalue, 3),
-                ", CHAR_ETAp2 = ",
-                np2,
-                the_ci,
-                lower,
-                ", ",
-                upper,
-                "]",
-                bf_out,
-                levene_post
-            )
-        } else {
-            out = paste0(
-                "F(",
-                df_n,
-                ",",
-                df_d,
-                ")",
-                " = ",
-                ro(F_val, 2),
-                ", p = ",
-                ro(pvalue, 3),
-                epsilon = eps_added,
-                ", CHAR_ETAp2 = ",
-                np2,
-                the_ci,
-                lower,
-                ", ",
-                upper,
-                "], CHAR_ETAG2 = ",
-                nG2,
-                bf_out,
-                " (",
-                f_name,
-                ")"
-            )
-        }
+        out = paste0(
+            "F(",
+            df_n,
+            ", ",
+            df_d,
+            ")",
+            " = ",
+            ro(F_val, 2),
+            ", p = ",
+            ro(pvalue, 3),
+            epsilon = eps_added,
+            ", CHAR_ETAp2 = ",
+            np2,
+            the_ci,
+            lower,
+            ", ",
+            upper,
+            "], CHAR_ETAG2 = ",
+            nG2,
+            bf_out,
+            " (",
+            f_name,
+            ")"
+        )
         if (hush == FALSE) {
             prnt(out)
         }
@@ -683,6 +826,11 @@ anova_apa = function(ezANOVA_out,
             bf = as.numeric(bf_val)
         )
     }
-    stat_list = c(stat_list, ez_anova = ezANOVA_out, bf_models = bf_models)
+    stat_list = list(effects = stat_list,
+                  ez_anova = ezANOVA_out,
+                  bf_models = bf_models,
+                  totals = dat_tot,
+                  residuals = resids,
+                  fitted = fitt)
     invisible(stat_list)
 }
