@@ -3,18 +3,23 @@
 #' @description Prints participant count, age mean and SD, and gender ratio,
 #'   from given dataset.
 #' @param data_per_subject Data frame from which demographics are to be
-#'   calculated. Must contain columns named as "\code{age}" and as
-#'   "\code{gender}" (or, alternatively, "\code{sex}"). The \code{age} column
-#'   must contain only numbers or \code{NA}, while \code{gender} column must
-#'   contain only \code{1} (= male) or \code{2} (= female), either as numbers or
-#'   as strings, or \code{NA}. Alternatively, male can be indicated, instead of
-#'   \code{1}, with the string \code{male} (or its abbreviations, e.g.
-#'   \code{m}), while in that case female can be indicated, instead of \code{2},
-#'   with the string \code{female} (or its abbreviations, e.g. \code{f} or
-#'   \code{fem} ). (Lettercases do not matter, e.g. \code{Male} or \code{MALE}
-#'   are both evaluated same as \code{male}.)
+#'   calculated. Should contain columns named as "\code{age}" and as
+#'   "\code{gender}" (or, alternatively, "\code{sex}"). Alternatively, these
+#'   columns can be specified via the \code{gender_col} and \code{age_col}
+#'   parameters. The \code{age} column must contain only numbers or \code{NA},
+#'   while \code{gender} column must contain only \code{1} (= male) or \code{2}
+#'   (= female), either as numbers or as strings, or \code{NA}. Alternatively,
+#'   male can be indicated, instead of \code{1}, with the string \code{male} (or
+#'   its abbreviations, e.g. \code{m}), while in that case female can be
+#'   indicated, instead of \code{2}, with the string \code{female} (or its
+#'   abbreviations, e.g. \code{f} or \code{fem} ). (Lettercases do not matter,
+#'   e.g. \code{Male} or \code{MALE} are both evaluated same as \code{male}.)
 #' @param group_by Optionally the name(s) of column(s) from the data frame
 #'   provided as \code{data_per_subject} to group by.
+#' @param gender_col Optionally the name of column from the data frame
+#'   that contains the gender (sex) information.
+#' @param age_col Optionally the name of column from the data frame
+#'   that contains the age information.
 #' @param percent Logical. If \code{TRUE}, gender ratios (and the
 #'   "unknown" ratios based on \code{NA} values) are presented as percentage. If
 #'   \code{FALSE}, they are presented as counts (i.e., numbers of subjects).
@@ -26,16 +31,17 @@
 #'   when there are no unknown cases, but both numbers are displayed when there
 #'   are any unknown cases.
 #'
-#'@details The function will first look for columns named precisely "\code{age}"
-#'and as "\code{gender}". If either is not found, the function looks for the
-#'same names but with any lettercase (e.g. "\code{AGE}" or "\code{Gender}"). If
-#'still no "\code{gender}" column is found, the function looks for "\code{sex}"
-#'column in the same manner. alternatively, "\code{sex}". (No columns found for
-#'either variable will result in error message.)
+#'@details If \code{gender_col} and/or \code{age_col} are not specified, the
+#'  function will first look for columns named precisely "\code{age}" and as
+#'  "\code{gender}". If either is not found, the function looks for the same
+#'  names but with any lettercase (e.g. "\code{AGE}" or "\code{Gender}"). If
+#'  still no "\code{gender}" column is found, the function looks for
+#'  "\code{sex}" column in the same manner. If no column is found for either,
+#'  all related values will be counted as "unknown" (\code{NA}).
 #'
-#'If \code{NA} values are found in either the \code{age} or \code{gender}
-#'column, the ratio (or count) of unknown cases will be displayed everywhere.
-#'Otherwise it will simply not be displayed anywhere.
+#'  If \code{NA} values are found in either the \code{age} or \code{gender}
+#'  column, the ratio (or count) of unknown cases will be displayed everywhere.
+#'  Otherwise it will simply not be displayed anywhere.
 #'
 #' @examples
 #' # below is an illustrative example dataset
@@ -64,10 +70,34 @@
 #' # again print demographics per "conditions":
 #' dems_neat(dat, group_by = 'conditions')
 #'
+#' # another dataset, with no "age"/"gender" columns
+#' dat = data.frame(
+#'     subject = c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+#'     conditions = c('x', 'y', 'x', 'y', 'y', 'x', 'x', 'x', 'y', 'x'),
+#'     geschlecht = c(2, 2, NA, NA, 1, 1, 1, 2, NA, NA),
+#'     alter = c(6, 7, 8.5, 6, 5, 16, NA, 16, 45, 77),
+#'     measure_y = c(83, 71, 111, 70, 92, 75, 110, 111, 110, 85),
+#'     stringsAsFactors = TRUE
+#' )
+#'
+#' # the folliwing will return "unknowns"
+#' dems_neat(dat, group_by = 'conditions')
+#'
+#' # age column specified
+#' dems_neat(dat, group_by = 'conditions', gender_col = 'geschlecht')
+#'
+#' # both columns specified
+#' dems_neat(dat,
+#'           group_by = 'conditions',
+#'           age_col = 'alter',
+#'           gender_col = 'geschlecht')
+#'
 #' @export
 
 dems_neat = function(data_per_subject,
                      group_by = NULL,
+                     gender_col = NULL,
+                     age_col = NULL,
                      percent = FALSE,
                      round_perc = 0,
                      show_fem = NULL) {
@@ -80,14 +110,20 @@ dems_neat = function(data_per_subject,
                   list(
                       val_arg(s_dat, c('df')),
                       val_arg(group_by, c('char', 'null')),
+                      val_arg(gender_col, c('char', 'null')),
+                      val_arg(age_col, c('char', 'null')),
                       val_arg(percent, c('bool'), 1),
                       val_arg(round_perc, c('num'), 1),
                       val_arg(show_fem, c('bool', 'null'), 1)
                   ))
-    if (!'age' %in% names(s_dat)) {
+    if (!is.null(age_col)) {
+        s_dat$age = s_dat[[age_col]]
+    } else if (!'age' %in% names(s_dat)) {
         names(s_dat)[tolower(names(s_dat)) == 'age'] = 'age'
     }
-    if (!'gender' %in% names(s_dat)) {
+    if (!is.null(gender_col)) {
+        s_dat$gender = s_dat[[gender_col]]
+    } else if (!'gender' %in% names(s_dat)) {
         names(s_dat)[tolower(names(s_dat)) == 'gender'] = 'gender'
         if (!'gender' %in% names(s_dat)) {
             names(s_dat)[names(s_dat) == 'sex'] = 'gender'
@@ -96,23 +132,15 @@ dems_neat = function(data_per_subject,
             }
         }
     }
+    no_age = FALSE
+    no_gender = FALSE
     if (!'age' %in% names(s_dat)) {
-        if (!'gender' %in% names(s_dat)) {
-            stop(
-                'The data frame must contain the columns "gender" (or "sex") and "age",',
-                ' but neither was found among the column names.'
-            )
-        } else {
-            stop(
-                'The data frame must contain the columns "gender" and "age".',
-                ' Column name "age" was not found.'
-            )
-        }
-    } else if (!'gender' %in% names(s_dat)) {
-        stop(
-            'The data frame must contain the columns "gender" (or "sex") and "age".',
-            ' Column name "gender" (or "sex") was not found.'
-        )
+        no_age = TRUE
+        s_dat$age = NA
+    }
+    if (!'gender' %in% names(s_dat)) {
+        no_gender = TRUE
+        s_dat$gender = NA
     }
     s_dat$gender = tolower(as.character(s_dat$gender))
     if (all(
@@ -154,7 +182,7 @@ dems_neat = function(data_per_subject,
     }
     gender$ratio = gender[['1']] / (gender[['1']] + gender[['2']]) * 100
     gender$neat_cond = row.names(gender)
-    
+
     age = do.call(data.frame,
                   stats::aggregate(s_dat$age, by = list(s_dat$neat_cond), function(x)
                       c(
