@@ -14,12 +14,27 @@
 #'@param pair Logical. If \code{TRUE}, all tests (t, BF, AUC) are conducted for
 #'  paired samples. If \code{FALSE} (default) for independent samples.
 #'@param nonparametric Logical (\code{FALSE} by default). If \code{TRUE}, uses
-#'  nonparametric (rank-based, "Wilcoxon") t-tests (including BFs; see Details).
+#'  nonparametric (rank-based, "Wilcoxon") t-tests (including BFs; see Notes).
 #'@param greater \code{NULL} or string (or number); optionally specifies
 #'  one-sided tests (t and BF): either "1" (\code{var1} mean expected to be
 #'  greater than \code{var2} mean) or "2" (\code{var2} mean expected to be
 #'  greater than \code{var1} mean). If \code{NULL} (default), the test is
 #'  two-sided.
+#'@param norm_tests Normality tests. Any or all of the following character input
+#'  is accepted (as a single string or a character vector; case-insensitive):
+#'  \code{"W"} (Shapiro-Wilk), \code{"K2"} (D'Agostino-Pearson), \code{"A2"}
+#'  (Anderson-Darling), \code{"JB"} (Jarque-Bera); see Notes. Two other options
+#'  are \code{"all"} (to choose all four previous tests at the same time) or
+#'  \code{"latent"} (default value; prints all tests only if
+#'  \code{nonparametric} is set to \code{FALSE} and any of the four tests gives
+#'  a p value below .05). Each normality test is performed for the difference
+#'  values between the two variables in case of paired samples, or for each of
+#'  the two variables for unpaired samples. Set to \code{"none"} to disable
+#'  (i.e., not to perform any normality tests).
+#'@param norm_plots If \code{TRUE} and \code{norm_tests} is not \code{"none"},
+#'  displays density, histogram, and Q-Q plots (and scatter plots for paired
+#'  tests) for each of the two variable (and differences for pairwise
+#'  observations, in case of paired samples).
 #'@param ci Numeric; confidence level for returned CIs for Cohen's d and AUC.
 #'@param bf_added Logical. If \code{TRUE} (default), Bayes factor is calculated
 #'  and displayed.
@@ -28,8 +43,10 @@
 #'@param bf_sample Number of samples used to estimate Bayes factor (\code{1000}
 #'  by default). More samples (e.g. \code{10000}) take longer time but give more
 #'  stable BF.
-#'@param auc_added Logical. If \code{TRUE}, AUC is calculated and displayed.
-#'  (\code{FALSE} by default.)
+#'@param auc_added Logical (\code{FALSE} by default). If \code{TRUE}, AUC is
+#'  calculated and displayed (including TPR and TNR, i.e., true positive and
+#'  true negative rates, i.e. sensitivity and specificity, using an optimal
+#'  cutoff value, i.e. threshold, that provides maximal TPR and TNR).
 #'@param r_added Logical. If \code{TRUE} (default), Pearson correlation is
 #'  calculated and displayed in case of paired comparison.
 #'@param for_table Logical. If \code{TRUE}, omits the confidence level display
@@ -47,12 +64,21 @@
 #'  (\code{var2} expected to be greater for 'cases' than \code{var1}). Not to be
 #'  confused with one-sided tests; see Details.
 #'@param hush Logical. If \code{TRUE}, prevents printing any details to console.
-#'@param plot_densities Logical. If \code{TRUE}, creates a density plot (i.e.,
-#'  \code{\link[stats:density]{Gaussian kernel density estimates}}) from the two
-#'  variables. When \code{auc_added} is \code{TRUE} (and the AUC is at least
-#'  .5), the best threshold value for classification (maximal differentiation
-#'  accuracy) is added to the plot as vertical line. (In case of multiple best
-#'  thresholds with identical overall accuracy, all are added.)
+#'@param plots Logical (or \code{NULL}). If \code{TRUE}, creates a combined
+#'  density plot (i.e., \code{\link[stats:density]{Gaussian kernel density
+#'  estimates}}) from the two variables. Includes dashed vertical lines to
+#'  indicate means of each of the two variables. If \code{nonparametric} is set
+#'  to \code{TRUE}, medians are calculated for these dashed lines instead of
+#'  means. When \code{auc_added} is \code{TRUE} (and the AUC is at least .5),
+#'  the best threshold value for classification (maximal differentiation
+#'  accuracy using Youden's index) is added to the plot as solid vertical line.
+#'  (In case of multiple best thresholds with identical overall accuracy, all
+#'  are added.) If \code{NULL}, same as if \code{TRUE} except that histogram is
+#'  added to the background.
+#'@param rug_size Numeric (\code{4} by default): size of the rug ticks below the
+#'  density plot. Set to \code{0} (zero) to omit rug plotting.
+#'@param aspect_ratio Aspect ratio of the plots: \code{1} (\code{1}/\code{1}) by
+#'  default. (Set to \code{NULL} for dynamic aspect ratio.)
 #'@param y_label String or \code{NULL}; the label for the \code{y} axis.
 #'  (Default: \code{"density estimate"}.)
 #'@param x_label String or \code{NULL}; the label for the \code{x} axis.
@@ -112,20 +138,34 @@
 #'  object, to be used e.g. with the \code{\link{roc_neat}} function. The other
 #'  is '\code{best_thresholds}', which contains the best threshold value(s) for
 #'  classification, along with corresponding specificity and sensitivity.
-#'  Finally, if \code{plot_densities} is \code{TRUE}, the plot is displayed as
-#'  well as returned as a \code{\link[ggplot2]{ggplot}} object, named
-#'  \code{density_plot}.
+#'  Finally, if \code{plots} is \code{TRUE} (or \code{NULL}), the plot is
+#'  displayed as well as returned as a \code{\link[ggplot2]{ggplot}} object,
+#'  named \code{t_plot}.
 #'
-#'@note The Welch's t-test is calculated via
+#'@note
+#'
+#'The Welch's t-test is calculated via
 #'\code{\link[stats:t.test]{stats::t.test}}.
 #'
+#'#'Normality tests are all calculated via
+#'\code{\link[PoweR:statcompute]{PoweR::statcompute}}, selected based on the
+#'recommendation of Lakens (2015), quoting Yap and Sim (2011, p. 2153): "If the
+#'distribution is symmetric with low kurtosis values (i.e. symmetric
+#'short-tailed distribution), then the D'Agostino-Pearson and Shapiro-Wilkes
+#'tests have good power. For symmetric distribution with high sample kurtosis
+#'(symmetric long-tailed), the researcher can use the JB, Shapiro-Wilkes, or
+#'Anderson-Darling test." See url{https://github.com/Lakens/perfect-t-test} for
+#'more details.
+#'
 #'Cohen's d and its confidence interval are calculated, using the t value, via
 #'\code{\link[MBESS:ci.smd]{MBESS::ci.smd}} for independent samples (as
 #'standardized mean difference) and via \code{\link[MBESS:ci.sm]{MBESS::ci.sm}}
 #'for paired samples (as standardized mean).
 #'
-#'The Bayes factor is calculated via
-#'\code{\link[BayesFactor:ttestBF]{BayesFactor::ttestBF}}.
+#'The parametric Bayes factor is calculated via
+#'\code{\link[BayesFactor:ttestBF]{BayesFactor::ttestBF}}. The nonparametric
+#'(rank-based) Bayes factor is a contribution by Johnny van Doorn; the original
+#'source code is available via \url{https://osf.io/gny35/}.
 #'
 #'The correlation and its CI are calculated via
 #'\code{\link[stats:cor.test]{stats::cor.test}}, and is always two-sided, always
@@ -136,7 +176,7 @@
 #'\code{\link[pROC:coords]{pROC::coords}} (\code{x = "best"}); both using the
 #'object \code{\link[pROC:roc]{pROC::roc}}.
 #'
-#'@references 
+#'@references
 #'
 #'Delacre, M., Lakens, D., & Leys, C. (2017). Why psychologists should by
 #'default use Welch's t-test instead of Student's t-test. International Review
@@ -146,6 +186,15 @@
 #'sciences: An R package. Behavior Research Methods, 39(4), 979-984.
 #'\doi{https://doi.org/10.3758/BF03192993}
 #'
+#'Lafaye de Micheaux, P., & Tran, V. A. (2016). PoweR: A Reproducible Research
+#'Tool to Ease Monte Carlo Power Simulation Studies for Goodness-of-fit Tests in
+#'R. Journal of Statistical Software, 69(3).
+#'\doi{https://doi.org/10.18637/jss.v069.i03}
+#'
+#'Lakens, D. (2015). The perfect t-test (version 1.0.0). Retrieved from
+#'https://github.com/Lakens/perfect-t-test.
+#'\doi{https://doi.org/10.5281/zenodo.17603}
+#'
 #'Robin, X., Turck, N., Hainard, A., Tiberti, N., Lisacek, F., Sanchez, J. C., &
 #'Muller, M. (2011). pROC: an open-source package for R and S+ to analyze and
 #'compare ROC curves. BMC bioinformatics, 12(1), 77.
@@ -156,6 +205,10 @@
 #'Spearman’s rho. Journal of Applied Statistics, 1–23.
 #'\doi{https://doi.org/10.1080/02664763.2019.1709053}
 #'
+#'Yap, B. W., & Sim, C. H. (2011). Comparisons of various types of normality
+#'tests. Journal of Statistical Computation and Simulation, 81(12), 2141–2155.
+#'\doi{https://doi.org/10.1080/00949655.2010.520163}
+#'
 #' @seealso \code{\link{corr_neat}}, \code{\link{roc_neat}}
 #' @examples
 #' # assign two variables (numeric vectors)
@@ -178,8 +231,10 @@ t_neat = function(var1,
                   pair = FALSE,
                   nonparametric = FALSE,
                   greater = NULL,
+                  norm_tests = 'latent',
+                  norm_plots = FALSE,
                   ci = NULL,
-                  bf_added = TRUE,
+                  bf_added = FALSE,
                   bf_rscale = sqrt(0.5),
                   bf_sample = 1000,
                   auc_added = FALSE,
@@ -190,7 +245,9 @@ t_neat = function(var1,
                   round_auc = 3,
                   auc_greater = '1',
                   hush = FALSE,
-                  plot_densities = FALSE,
+                  plots = FALSE,
+                  rug_size = 4,
+                  aspect_ratio = 1,
                   y_label = "density estimate",
                   x_label = "\nvalues",
                   factor_name = NULL,
@@ -204,6 +261,8 @@ t_neat = function(var1,
             val_arg(pair, c('bool'), 1),
             val_arg(nonparametric, c('bool'), 1),
             val_arg(greater, c('null', 'char'), 1, c('1', '2')),
+            val_arg(norm_tests, c('char')),
+            val_arg(norm_plots, c('bool'), 1),
             val_arg(ci, c('null', 'num'), 1),
             val_arg(bf_added, c('bool'), 1),
             val_arg(bf_rscale, c('num'), 1),
@@ -216,7 +275,9 @@ t_neat = function(var1,
             val_arg(round_auc, c('num'), 1),
             val_arg(auc_greater, c('char'), 1, c('1', '2')),
             val_arg(hush, c('bool'), 1),
-            val_arg(plot_densities, c('bool'), 1),
+            val_arg(plots, c('bool', 'null'), 1),
+            val_arg(rug_size, c('num'), 1),
+            val_arg(aspect_ratio, c('num', 'null'), 1),
             val_arg(y_label, c('null', 'char'), 1),
             val_arg(x_label, c('null', 'char'), 1),
             val_arg(factor_name, c('null', 'char'), 1),
@@ -250,6 +311,21 @@ t_neat = function(var1,
             message("NA values omitted.")
         }
     }
+    if (norm_tests != 'none' &
+        hush == FALSE) {
+        norm_tests_in(
+            var1 = var1,
+            var2 = var2,
+            pair = pair,
+            norm_tests = norm_tests,
+            alpha = 0.05,
+            hush = FALSE,
+            plots = norm_plots,
+            tneet = TRUE,
+            nonparametric = nonparametric,
+            aspect_ratio = 1
+        )
+    }
     descr_1 = paste0(ro(mean(var1), round_descr),
                      "CHAR_PLUSMIN",
                      ro(stats::sd(var1), round_descr))
@@ -279,7 +355,7 @@ t_neat = function(var1,
             cat('', fill = TRUE)
         }
     }
-    if (pair == TRUE & r_added == TRUE) {
+    if (pair == TRUE & r_added == TRUE & hush == FALSE) {
         if (nonparametric == TRUE) {
             cat("Spearman's rank correlation: ")
             corr_neat(var1,
@@ -512,32 +588,39 @@ t_neat = function(var1,
             levels = c(0, 1),
             direction =  auc_dir
         ) # v1 larger
-        if (hush == FALSE) {
-            show_auc(
-                theroc = the_roc,
-                ci = ci,
-                round_to = round_auc,
-                for_table = for_table
-            )
-        }
         youdn = pROC::coords(the_roc, x = "best", ret = "youden")
         if (class(youdn) == "data.frame") {
             maxyouden = as.numeric(youdn$youden[1])-1
         } else {
-            maxyouden =as.numeric(youdn[1])-1
+            maxyouden = as.numeric(youdn[1])-1
         }
         bestacc = pROC::coords(the_roc, x = "best", ret = "accuracy")
         if (class(bestacc) == "data.frame") {
             max_acc = as.numeric(bestacc$accuracy[1])
         } else {
-            max_acc =as.numeric(bestacc[1])
+            max_acc = as.numeric(bestacc[1])
         }
         best_coords = pROC::coords(the_roc, x = "best")
         the_auc = pROC::auc(the_roc)
-        if (class(best_coords) == "matrix") {
-            plot_thres = best_coords["threshold",]
+        if (class(best_coords) == "data.frame") {
+            plot_thres = as.numeric(best_coords$threshold)[1]
+            best_tp = as.numeric(best_coords$sensitivity)[1]
+            best_fp = as.numeric(best_coords$specificity)[1]
         } else {
-            plot_thres = best_coords["threshold"]
+            plot_thres = as.numeric(best_coords["threshold"])[1]
+            best_tp = as.numeric(best_coords["sensitivity"])[1]
+            best_fp = as.numeric(best_coords["specificity"])[1]
+        }
+        if (hush == FALSE) {
+            show_auc(
+                theroc = the_roc,
+                ci = ci,
+                round_to = round_auc,
+                for_table = for_table,
+                thres = plot_thres,
+                best_tp = best_tp,
+                best_fp = best_fp
+            )
         }
         plot_thres = plot_thres[!plot_thres %in% c(-Inf, Inf)]
     } else {
@@ -548,7 +631,7 @@ t_neat = function(var1,
         best_coords = NULL
         plot_thres = NULL
     }
-    if (plot_densities == TRUE) {
+    if (is.null(plots) || plots == TRUE) {
         the_plot = plot_dens(
             v1 = var1,
             v2 = var2,
@@ -557,7 +640,11 @@ t_neat = function(var1,
             thres = plot_thres,
             factor_name = factor_name,
             var_names = var_names,
-            reverse = reverse
+            reverse = reverse,
+            hist = plots,
+            rug_size = rug_size,
+            aspect_ratio = aspect_ratio,
+            nonparam = nonparametric
         )
         graphics::plot(the_plot)
     } else {
@@ -575,7 +662,7 @@ t_neat = function(var1,
         ),
         roc_obj = the_roc,
         best_thresholds = best_coords,
-        density_plot = the_plot
+        t_plot = the_plot
     ))
 }
 
@@ -588,34 +675,67 @@ plot_dens = function(v1,
                      thres,
                      factor_name,
                      var_names,
-                     reverse) {
+                     reverse,
+                     hist,
+                     rug_size,
+                     aspect_ratio,
+                     nonparam) {
     dens_dat = data.frame(vals = c(v1, v2),
                           facts = c(rep(var_names[1], length(v1)),
                                     rep(var_names[2], length(v2))))
     if (reverse == TRUE) {
         dens_dat$facts =  factor(dens_dat$facts, levels = c(var_names[2], var_names[1]))
-        start_color = 0.8
-        end_color = 0.2
+        colrs = c('#006600', '#b3b3ff')
     } else {
-        dens_dat$facts =  factor(dens_dat$facts, levels = var_names)
-        start_color = 0.2
-        end_color = 0.8
+        colrs = c('#b3b3ff', '#006600')
+    }
+    if (is.null(hist)) {
+        max_1 = (max(v1) - min(v1)) / 10
+        max_2 = (max(v2) - min(v2)) / 10
+        freed1 = 2 * stats::IQR(v1) / (length(v1) ^ (1 / 3))
+        freed2 = 2 * stats::IQR(v2) / (length(v2) ^ (1 / 3))
+        my_binwidth = min(max_1, max_2, freed1, freed2)
+        the_plot = ggplot(dens_dat,
+                        aes(
+                            x = .data$vals,
+                            fill = .data$facts,
+                            color = .data$facts
+                        )) +
+            geom_histogram(
+                aes(y = .data$..count..),
+                alpha = 0.1,
+                binwidth = my_binwidth,
+                color = 'darkgray',
+                position = "identity"
+            ) +
+            geom_density(aes(y = .data$..count.. * my_binwidth), alpha = 0.3)
+    } else {
+        the_plot = ggplot(dens_dat,
+                        aes(
+                            x = .data$vals,
+                            fill = .data$facts,
+                            color = .data$facts
+                        )) +
+            geom_density(aes(y = .data$..count.. / sum(.data$..count..)),
+                         alpha = 0.3)
+    }
+    if (rug_size > 0) {
+        yrange = ggplot_build(the_plot)$layout$panel_params[[1]]$y.range
+        hght = yrange[2] - yrange[1]
+        dens_dat$ypos <- -hght / 20
+        dens_dat$ypos[dens_dat$facts == var_names[2]] <-
+            -(2 * hght / 20)
+        the_plot = the_plot + geom_point(
+            aes(
+                x = .data$vals,
+                y = dens_dat$ypos,
+                colour = .data$facts
+            ),
+            alpha = 1,
+            shape = '|',
+            size = rug_size
+        )
     }
-    the_plot = ggplot(data = dens_dat, aes(x = dens_dat$vals,
-                                           fill = dens_dat$facts)) + geom_density(alpha = 0.4, trim = FALSE) +
-        theme_classic() +
-        theme(text = element_text(family = "serif", size = 17),
-              panel.border = element_rect(fill = NA)) +
-        scale_fill_grey(name = factor_name,
-                        start = start_color,
-                        end = end_color) +
-        geom_vline(
-            xintercept = c(mean(v1), mean(v2)),
-            color = "#777777",
-            linetype = "dashed",
-            size = 0.5
-        )  +
-        ylab(y_label) + xlab(x_label)
     if (!is.null(thres)) {
         the_plot = the_plot +
             geom_vline(
@@ -625,5 +745,24 @@ plot_dens = function(v1,
                 size = 0.5
             )
     }
-    return(the_plot)
+    if (nonparam == TRUE) {
+        xfunc = stats::median
+    } else {
+        xfunc = mean
+    }
+    return(
+        the_plot + scale_fill_manual(values = c('#004d00', '#8080ff'),
+                                     name = factor_name) +
+            scale_color_manual(values = c('#004d00', '#8080ff'), guide = FALSE) +
+            geom_vline(
+                xintercept = c(xfunc(v1), xfunc(v2)),
+                color = "#777777",
+                linetype = "dashed",
+                size = 0.5
+            ) + theme_bw() +
+            ylab(y_label) + xlab(x_label) + theme(
+                aspect.ratio = aspect_ratio,
+                text = element_text(family = "serif", size = 17)
+            )
+    )
 }
